/* The main window instance */

const { ipcRenderer } = require('electron'),
    logger = require('../common/logger.js'),
    ejs = require('ejs'),
    e = require('express');

/* Global variables */
var countKaraInQueue = 0;
var currentSong = -1;
var currentSongElement;
var currentSongPos = 0;
var dragCounter = 0;
var leavedElement;
var isDnDFromDB = false;
var isPlaying = false;
var currentPlaylist = '';
var isQueueView = true;
var clearMenuQueue;
var clearMenuPlaylist;

function updatePlayPauseButton(state) {
    logger.debug('instance', `State was ${state}`);
    if (state === 'play') {
        /* Play */
        isPlaying = true;
        $('#commandPlay').children().addClass('fa-pause');
        $('#commandPlay').children().removeClass('fa-play');
    } else {
        /* Pause or Stop */
        isPlaying = false;
        $('#commandPlay').children().addClass('fa-play');
        $('#commandPlay').children().removeClass('fa-pause');
    }
}

function addIpcToButton(btnId, ipc) {
    document.getElementById(btnId).addEventListener('click', () => {
        ipc.forEach(ipc => {
            ipcRenderer.send(ipc);
        });
    });
}

function autoFillSettings() {
    $('#inputHost').val(config.content.lektord.host);
    $('#inputPort').val(config.content.lektord.port);
    $('#inputAttachMode').prop('checked', true);
    $('#inputDbPath').val(config.content.database.path);
    $('#inputLogLektord').prop('checked', config.content.log.lektord);
    $(`#log${config.content.log.level.toUpperCase()}`).prop('checked', true);
    $('#inputKurisuUrl').val(config.content.kurisu.url);
    $('#inputClientPort').val(config.content.client.port);
}

function flushFillSettings() {
    const set = {
        kurisuurl: $('#inputKurisuUrl').val(),
        clientport: $('#inputClientPort').val(),
        host: $('#inputHost').val(),
        port: $('#inputPort').val(),
        attachmode: $('#inputAttachMode').prop('checked'),
        db: $('#inputDbPath').val(),
        loglektord: $('#inputLogLektord').prop('checked'),
        loglevel: {
            ERROR: $('#logERROR').prop('checked'),
            WARNING: $('#logWARNING').prop('checked'),
            INFO: $('#logINFO').prop('checked'),
            DEBUG: $('#logDEBUG').prop('checked'),
        },
    };
    ipcRenderer.send('set-settings', set);
}

window.onload = () => {
    /* Set things up for the settings modal*/
    var modal = document.getElementById('modal-settings');
    var btn = document.getElementById('btn-settings');
    var span = document.getElementsByClassName('modal-close')[0];

    btn.onclick = () => (modal.style.display = 'block');
    span.onclick = () => {
        modal.style.display = 'none';
        flushFillSettings();
    };
    window.onclick = event => {
        if (event.target == modal) {
            modal.style.display = 'none';
        }
    };

    /* Add IPC to button list */
    addIpcToButton('closeButton', ['close-app']);

    addIpcToButton('commandPlay', ['cmd-play']);
    addIpcToButton('commandStop', ['cmd-stop']);
    addIpcToButton('commandClear', ['cmd-clear']);

    addIpcToButton('updateDB', ['update-database']);
    addIpcToButton('dryUpdate', ['dry-update-database']);

    addIpcToButton('open-kurisu-view', ['toggle-kurisu']);
    addIpcToButton('open-user-view', ['toggle-client-view']);
    addIpcToButton('open-log-view', ['toggle-log-view']);
    addIpcToButton('newPlaylist', ['new-playlist']);

    var switchQP = document.getElementById('switchQueuePlaylist');
    switchQP.innerText = 'View playlist';
    switchQP.onclick = () => switchQueuePlaylist();

    document.getElementById('addResearchToQueue').onclick = () => {
        ipcRenderer.send('add-kara-queue-search-result', `${$('#filterInput').val()}`);
    };

    $('#openMdtView').click(() => $('#mdt-view').toggle());

    ipcRenderer.send('verify-lektord');
    logger.debug('instance', 'Window loaded');
    document.getElementById('filterInput').addEventListener('input', e => {
        /* On 'Return'. */
        logger.debug('instance', `Send filter for: ${$('#filterInput').val()}`);
        ipcRenderer.send('reload-db-request', {
            search: `${$('#filterInput').val()}`,
        });
    })

    document.addEventListener(
        'keydown',
        e => {
            if (e.ctrlKey && e.keyCode == 70) {
                /* ctrl + F */
                logger.debug('instance', 'Focus find bar');
                $('#filterInput').focus();
            } else if (e.ctrlKey && e.keyCode == 67) {
                /* ctrl + C */
                logger.debug('instance', `Focus current kara in queue, id is "#pos-${currentSong + 1}"`);
                $('#panelRight').animate(
                    {
                        scrollTop: 0,
                    },
                    0
                );
                var offset = $(window).height() / 2;
                $('#panelRight').animate(
                    {
                        scrollTop:
                            $(`#pos-${currentSong + 1}`)
                                .parent()
                                .offset().top - offset,
                    },
                    0
                );
            }
        },
        false
    );

    /* Autofill settings */
    autoFillSettings();

    /* Right click menus */
    $('#panelRight')
        .on('contextmenu', e => {
            var top = e.pageY - 10;
            var left = e.pageX - 90;
            if (isQueueView) {
                $('#context-menu-right')
                    .css({
                        display: 'block',
                        top: top,
                        left: left,
                    })
                    .addClass('show');
            } else {
                $('#context-menu-right-playlists')
                    .css({
                        display: 'block',
                        top: top,
                        left: left,
                    })
                    .addClass('show');
            }
            return false; /* blocks default Webbrowser right click menu */
        })
        .on('click', () => {
            $('#context-menu-right').removeClass('show').hide();
            $('#context-menu-right-playlists').removeClass('show').hide();
        });

    clearMenuQueue = () => $('#context-menu-right').removeClass('show').hide();
    clearMenuPlaylist = () => $('#context-menu-right-playlists').removeClass('show').hide();
    $('#queue-play').click(() => {
        clearMenuQueue();
        logger.debug('instance', 'Queue play');
        ipcRenderer.send('cmd-unpause');
    });
    $('#queue-pause').click(() => {
        clearMenuQueue();
        logger.debug('instance', 'Queue pause');
        ipcRenderer.send('cmd-pause');
    });
    $('#queue-recenter').click(() => {
        clearMenuQueue();
        if (currentSongElement) {
            currentSongElement.scrollIntoView(true);
        }
        logger.debug('instance', 'Queue recenter');
    });
    $('#queue-find').click(() => {
        clearMenuQueue();
        logger.debug('instance', 'Queue find');
    });
    $('#queue-shuffle').click(() => {
        clearMenuQueue();
        logger.debug('instance', 'Queue shuffle');
        ipcRenderer.send('cmd-shuffle');
    });
    $('#queue-clear').click(() => {
        clearMenuQueue();
        logger.debug('instance', 'Queue clear');
        ipcRenderer.send('cmd-clear');
    });
    $('#playlist-addToQueue').click(() => {
        clearMenuPlaylist();
        logger.debug('instance', 'Playlist add to queue');
        ipcRenderer.send('add-current-playlist-queue');
    });
    $('#playlist-clear').click(() => {
        clearMenuPlaylist();
        logger.debug('instance', 'Playlist clear');
        ipcRenderer.send('clear-playlist');
    });
    $('#playlist-delete').click(() => {
        clearMenuPlaylist();
        logger.debug('instance', 'Playlist delete');
        ipcRenderer.send('delete-playlist');
    });

    var playlistSelect = document.getElementById('playlist-selector');

    playlistSelect.onchange = () => {
        currentPlaylist = playlistSelect.value;
        ipcRenderer.send('get-playlist-data', playlistSelect.value);
    };
};

/* Create the left panel */
ipcRenderer.on('reload-db-responce', (event, arg) => {
    logger.debug('instance', `Web page got reload-db`);
    document.getElementById('panelLeft').innerHTML = '';
    arg.forEach(kara => {
        if (kara.available == 1) {
            ejs.renderFile(__dirname + '/views/karaDBListItem.ejs', { kara: kara }, (err, data) => {
                if (err) logger.error('instance', err);
                $('#panelLeft').append(data);
            });
        } else {
            ejs.renderFile(__dirname + '/views/karaDBUnavailableListItem.ejs', { kara: kara }, (err, data) => {
                if (err) logger.error('instance', err);
                $('#panelLeft').append(data);
            });
        }
    });
    [].forEach.call(document.querySelectorAll('#panelLeft .karaCard'), addDBKaraEventHandlers);
    if (!isQueueView) {
        [].forEach.call(document.querySelectorAll('#panelLeft .karaCard'), element => {
            if (element.getElementsByClassName('unavailable').length == 0) {
                element.getElementsByClassName('karaAddBtn')[0].hidden = true;
                element.getElementsByClassName('karaInsertBtn')[0].hidden = true;
                element.getElementsByClassName('karaAddPlaylistBtn')[0].hidden = false;
            }
        });
    }
});

/* Create the right panel: the queue */
ipcRenderer.on('reload-queue-responce', (event, arg) => {
    logger.debug('instance', `Web page got reload-queue`);
    countKaraInQueue = 0;
    if (isQueueView) {
        document.getElementById('panelRight').innerHTML = '';
        arg.forEach(kara => {
            countKaraInQueue++;
            ejs.renderFile(__dirname + '/views/karaQueueListItem.ejs', { kara: kara }, (err, data) => {
                if (err) logger.error('instance', err);
                $('#panelRight').append(data);
            });
        });
        [].forEach.call(document.querySelectorAll('#panelRight .karaCard'), addQueueKaraEventHandlers);
    } else {
        document.getElementById('panelBuffer').innerHTML = '';
        arg.forEach(kara => {
            countKaraInQueue++;
            ejs.renderFile(__dirname + '/views/karaQueueListItem.ejs', { kara: kara }, (err, data) => {
                if (err) logger.error('instance', err);
                $('#panelBuffer').append(data);
            });
        });
    }
});

ipcRenderer.on('playlist-data-responce', (event, karas) => {
    var playListPanel = document.getElementById('panelRight');
    if (!isQueueView) {
        document.getElementById('panelRight').innerHTML = '';
        karas.forEach(kara => {
            ejs.renderFile(__dirname + '/views/karaPlaylistListItem.ejs', { kara: kara }, (err, data) => {
                if (err) logger.error('instance', err);
                $('#panelRight').append(data);
            });
        });
        [].forEach.call(document.querySelectorAll('#panelRight .karaCard'), addPlayListKaraEventHandlers);
    } else {
        document.getElementById('panelBuffer').innerHTML = '';
        karas.forEach(kara => {
            ejs.renderFile(__dirname + '/views/karaPlaylistListItem.ejs', { kara: kara }, (err, data) => {
                if (err) logger.error('instance', err);
                $('#panelBuffer').append(data);
            });
        });
    }
});
ipcRenderer.on('send-lektord-is-attached', (event, arg) => {
    logger.debug('instance', `Lektord is in attached mode? ${arg}`);
});

ipcRenderer.on('send-state', (event, state) => updatePlayPauseButton(state));

ipcRenderer.on('send-song-mdt', (event, mdt) => {
    document.getElementById('mdt-source').innerHTML = mdt.Source;
    document.getElementById('mdt-title').innerHTML = `${mdt.Type} - ${mdt.Title}`;
    document.getElementById('mdt-text').innerHTML = `Karaoke by ${mdt.Author}${
        isPlaying ? ' is currently playing' : ' is loaded but not playing'
    }`;

    document.getElementById('mdt-other').innerHTML = `
    <span class="badge bg-light text-dark">${mdt.Category}</span>
    <span class="badge bg-light text-dark">${mdt.Language}</span>
    `;
});

ipcRenderer.on('send-song-time-data', (event, timeData) => {
    document.getElementById('progressBar').style.width = `${
        (timeData.elapsed / timeData.total) * document.documentElement.clientWidth
    }px`;
    if (timeData.state == 'play' || timeData.state == 'pause') {
        if (currentSong != timeData.song) {
            if (currentSongElement) {
                currentSongElement.style.background = '#4e5d6c';
            }
            tmp = document.getElementById(`pos-${currentSongPos.toString()}`);
            if (tmp) tmp.parentElement.style.background = '#4e5d6c';
            currentSongPos = parseInt(timeData.song) + 1;
            currentSongElement = document.getElementById(`pos-${currentSongPos.toString()}`).parentElement;
            currentSong = timeData.song;
            currentSongElement.style.background = '#6b7d8e';
            currentSongElement.scrollIntoView(true);
        }
        document.getElementById(`pos-${currentSongPos.toString()}`).parentElement.style.background = '#6b7d8e';
    } else {
        if (currentSongElement) currentSongElement.style.background = '#4e5d6c';
        currentSong = document.getElementById(`pos-${currentSongPos.toString()}`);
        if (currentSong) currentSongs.parentElement.style.background = '#4e5d6c';
    }
});

ipcRenderer.on('send-runnings', (event, arg) => {
    $('#running-lektord').prop('checked', arg.lektord);
    $('#running-klkt').prop('checked', arg.klkt);
});

ipcRenderer.on('send-lektord-is-attached', (event, arg) => {
    logger.debug('instance', `Lektord is in attached mode? ${arg}`);
});

ipcRenderer.on('check-db-updated', (event, arg) => {
    ipcRenderer.send('reload-db-request');
});

/*
 * Drag'n'drop functions
 */

function addDBKaraEventHandlers(element) {
    if (element.getElementsByClassName('karaDownloadBtn').length == 0) {
        element.addEventListener('dragstart', onDragStartDB, false);
        element.addEventListener('dragend', onDragEnd, false);
        element.getElementsByClassName('karaAddBtn')[0].addEventListener('click', () =>
            ipcRenderer.send('add-kara-queue-id', {
                id: element.getElementsByClassName('karaID')[0].innerText,
            })
        );
        element.getElementsByClassName('karaInsertBtn')[0].addEventListener('click', () =>
            ipcRenderer.send('insert-kara-queue-id', {
                id: element.getElementsByClassName('karaID')[0].innerText,
            })
        );
        element.getElementsByClassName('karaAddPlaylistBtn')[0].addEventListener('click', () =>
            ipcRenderer.send('add-kara-playlist-id', {
                id: element.getElementsByClassName('karaID')[0].innerText,
            })
        );
    } else {
        element.getElementsByClassName('karaDownloadBtn')[0].addEventListener('click', () =>
            ipcRenderer.send('download-kara-id', {
                id: element.getElementsByClassName('karaID')[0].innerText,
            })
        );
    }
}

function addQueueKaraEventHandlers(element) {
    element.addEventListener('dragstart', onDragStartQueue, false);
    element.addEventListener('dragenter', onDragEnter, false);
    element.addEventListener('dragover', onDragOver, false);
    element.addEventListener('dragleave', ondragLeave, false);
    element.addEventListener('drop', onDrop, false);
    element.addEventListener('dragend', onDragEnd, false);
    element.getElementsByClassName('karaDeleteBtn')[0].addEventListener('click', () =>
        ipcRenderer.send('delete-kara-queue-pos', {
            position: element.getElementsByClassName('karaPosition')[0].innerText,
        })
    );
    element.getElementsByClassName('karaPlayBtn')[0].addEventListener('click', () => {
        ipcRenderer.send('play-kara-queue-pos', {
            position: element.getElementsByClassName('karaPosition')[0].innerText,
        });
    });
}

function addPlayListKaraEventHandlers(element) {
    element.getElementsByClassName('karaDeleteBtn')[0].addEventListener('click', () => {
        ipcRenderer.send('del-kara-playlist-id', {
            id: element.getElementsByClassName('karaID')[0].innerText,
        });
    });
    element.getElementsByClassName('karaAddBtn')[0].addEventListener('click', () =>
        ipcRenderer.send('add-kara-queue-id', {
            id: element.getElementsByClassName('karaID')[0].innerText,
        })
    );
    element.getElementsByClassName('karaInsertBtn')[0].addEventListener('click', () =>
        ipcRenderer.send('insert-kara-queue-id', {
            id: element.getElementsByClassName('karaID')[0].innerText,
        })
    );
}

function onDragStartDB(event) {
    isDnDFromDB = true;
    event.dataTransfer.effectAllowed = 'move';
    event.currentTarget.style.opacity = 0.4;
    event.dataTransfer.setData('text/plain', event.currentTarget.getElementsByClassName('karaID')[0].innerText);
}

function onDragStartQueue(event) {
    event.dataTransfer.effectAllowed = 'move';
    event.currentTarget.style.opacity = 0.4;
    event.dataTransfer.setData('text/plain', event.currentTarget.getElementsByClassName('karaPosition')[0].innerText);
}

function onDragEnter(event) {
    if (event.currentTarget) {
        if (leavedElement != event.currentTarget && leavedElement) {
            leavedElement.style.borderTop = ''; // style is undefined !
        }
        event.currentTarget.style.borderTop = '3px solid grey';
        dragCounter++;
    }
}

function onDragOver(event) {
    if (event.preventDefault) {
        event.preventDefault();
    }
    event.dataTransfer.dropEffect = 'move';
}

function ondragLeave(event) {
    dragCounter--;
    leavedElement = event.currentTarget;
    if (dragCounter == 0) {
        event.currentTarget.style.borderTop = '';
    }
}

function onDrop(event) {
    event.currentTarget.style.borderTop = '';
    if (isDnDFromDB) {
        ipcRenderer.send('add-kara-queue-pos', {
            id: event.dataTransfer.getData('text/plain'),
            position: event.currentTarget.getElementsByClassName('karaPosition')[0].innerText,
            queueSize: countKaraInQueue,
        });
    } else {
        ipcRenderer.send('queue-moved-kara', {
            from: event.dataTransfer.getData('text/plain'),
            to: event.currentTarget.getElementsByClassName('karaPosition')[0].innerText,
        });
    }
    isDnDFromDB = false;
    return false;
}

function onDragEnd(event) {
    event.currentTarget.style.opacity = 1;
    dragCounter = 0;
}

function switchQueuePlaylist() {
    isQueueView = !isQueueView;
    var switchQP = document.getElementById('switchQueuePlaylist');
    var tempHTML = document.getElementById('panelRight').innerHTML;
    document.getElementById('panelRight').innerHTML = document.getElementById('panelBuffer').innerHTML;
    document.getElementById('panelBuffer').innerHTML = tempHTML;
    if (isQueueView) {
        switchQP.innerText = 'View playlist';
        [].forEach.call(document.querySelectorAll('#panelLeft .karaCard'), element => {
            if (element.getElementsByClassName('unavailable').length == 0) {
                element.getElementsByClassName('karaAddBtn')[0].hidden = false;
                element.getElementsByClassName('karaInsertBtn')[0].hidden = false;
                element.getElementsByClassName('karaAddPlaylistBtn')[0].hidden = true;
            }
        });
        [].forEach.call(document.querySelectorAll('#panelRight .karaCard'), addQueueKaraEventHandlers);
        clearMenuPlaylist();
    } else {
        switchQP.innerText = 'View queue';
        [].forEach.call(document.querySelectorAll('#panelLeft .karaCard'), element => {
            if (element.getElementsByClassName('unavailable').length == 0) {
                element.getElementsByClassName('karaAddBtn')[0].hidden = true;
                element.getElementsByClassName('karaInsertBtn')[0].hidden = true;
                element.getElementsByClassName('karaAddPlaylistBtn')[0].hidden = false;
            }
        });
        [].forEach.call(document.querySelectorAll('#panelRight .karaCard'), addPlayListKaraEventHandlers);
        clearMenuQueue();
    }
}

ipcRenderer.on('playlists-updated', (event, playlists) => {
    if (!playlists) {
        return;
    }
    var selector = document.getElementById('playlist-selector');
    var currentChoice = selector.value;
    if (!currentChoice || !playlists.includes(currentChoice)) {
        selector.innerHTML = '<option selected disabled> Select a playlist</option>';
    } else {
        selector.innerHTML = '<option disabled> Select a playlist</option>';
    }
    playlists.forEach(playlist => {
        if (playlist == currentChoice) {
            optionHtml = `<option selected value="${playlist}">${playlist}</option>`;
        } else {
            optionHtml = `<option value="${playlist}">${playlist}</option>`;
        }
        selector.innerHTML = `
        ${selector.innerHTML}
        ${optionHtml}
        `;
    });
});
